import 'dart:ui';

import 'package:scanbot_sdk/json/common_data.dart';
import 'package:json_annotation/json_annotation.dart';
import 'package:scanbot_sdk/sdk_utils.dart';

part 'data_configuration_json.g.dart';

@JsonSerializable(
    includeIfNull: false, createFactory: false, explicitToJson: true)
class TextDataScannerJsonConfiguration {
  ///A version of the configuration object
  String? version;

  ///The configuration object should be applied for this screen
  String? screen;

  ///The preferred camera module (default - BACK)
  CameraModule? cameraModule;

  @JsonKey(fromJson: colorFromString, toJson: toColorString)

  ///Background color outside of the finder window.
  Color? cameraOverlayColor;

  ///Title of the cancel button.
  String? cancelButtonTitle;

  ///Title of the button that opens the screen where the user can allow the usage of the camera by the app.
  String? enableCameraButtonTitle;

  ///Text that will be displayed when the app is not allowed to use the camera, prompting the user to enable the usage of the camera.
  String? enableCameraExplanationText;

  @JsonKey(fromJson: colorFromString, toJson: toColorString)

  ///Foreground color of the detection overlay.
  Color? finderLineColor;

  ///Width of finder frame border. Default is 2.
  double? finderLineWidth;

  @JsonKey(fromJson: colorFromString, toJson: toColorString)

  ///Foreground color of the description label.
  Color? finderTextHintColor;

  ///Controls whether the flash toggle button is hidden or not.
  bool? flashButtonHidden;

  ///Title of the flash toggle button.
  String? flashButtonTitle;

  ///Controls whether the flash should be initially enabled. The default value is FALSE.
  bool? flashEnabled;

  ///UI Interface orientation lock mode
  OrientationLockMode? orientationLockMode;

  @JsonKey(fromJson: colorFromString, toJson: toColorString)

  ///The background color of the top toolbar.
  Color? topBarBackgroundColor;

  @JsonKey(fromJson: colorFromString, toJson: toColorString)

  ///The color of all active toggle buttons in the toolbar.
  Color? topBarButtonsActiveColor;

  @JsonKey(fromJson: colorFromString, toJson: toColorString)

  ///The color of all inactive toggle buttons in the toolbar.
  Color? topBarButtonsInactiveColor;

  @JsonKey(fromJson: colorFromString, toJson: toColorString)

  ///The color of the highlight of the wordboxes founded on the frame
  Color? wordBoxHighlightColor;

  ///If the wordboxes should be highlighted when were found on the frame
  bool? wordBoxHighlightEnabled;

  ///Minimum number of accumulated frames that have equal result
  int? minimumNumberOfRequiredFramesWithEqualRecognitionResult;

  ///Maximum number of accumulated frames to inspect before actual result is returned
  int? maximumNumberOfAccumulatedFrames;

  ///Languages on which the recognition will be run. ISO 639-1 or RFC4646 for special languages that are not available for ISO 639-1
  List<String>? supportedLanguages;

  ///The image will be downscaled to the given value during the processing. If the quality of the scanning is not enough, try to increase the limit. This variable affects the performance of the scanning process. Higher the size limit - better the quality, but the recognition is slower. 0 - do not rescale.
  int? ocrResolutionLimit;

  ///Controls whether buttons should use all capitals style, as defined by the Android Material Design. Defaults to TRUE. Android only.
  bool? useButtonsAllCaps;

  ///If `true`, replaces the cancel button in the top bar with a back arrow icon. The default value is FALSE. Android only.
  bool? replaceCancelButtonWithIcon;

  ///Preview mode of the camera. FILL_IN or FIT_IN. Default is FILL_IN. Android only
  CameraPreviewMode? cameraPreviewMode;

  ///Configuration for the scanned item
  TextDataScannerStep? textDataScannerStep;

  TextDataScannerJsonConfiguration({
    this.version,
    this.screen,
    this.cameraModule,
    this.cameraOverlayColor,
    this.cancelButtonTitle,
    this.enableCameraButtonTitle,
    this.enableCameraExplanationText,
    this.finderLineColor,
    this.finderLineWidth,
    this.finderTextHintColor,
    this.flashButtonHidden,
    this.flashButtonTitle,
    this.flashEnabled,
    this.orientationLockMode,
    this.topBarBackgroundColor,
    this.topBarButtonsActiveColor,
    this.topBarButtonsInactiveColor,
    this.wordBoxHighlightColor,
    this.wordBoxHighlightEnabled,
    this.minimumNumberOfRequiredFramesWithEqualRecognitionResult,
    this.maximumNumberOfAccumulatedFrames,
    this.supportedLanguages,
    this.ocrResolutionLimit,
    this.useButtonsAllCaps,
    this.replaceCancelButtonWithIcon,
    this.cameraPreviewMode,
    this.textDataScannerStep,
  });

  Map<String, dynamic> toJson() =>
      _$TextDataScannerJsonConfigurationToJson(this);
}

/// Configuration for the scanned item
@JsonSerializable(includeIfNull: false, explicitToJson: true)
class TextDataScannerStep {
  /// User guidance hint text.
  String guidanceText;

  /// Validation pattern to automatically validate recognized text. '?' = any character, '#' - any digit, all other characters represent themselves. An empty string or nil value will disable the validation pattern.
  String pattern;

  /// If set to TRUE pattern validation also validates successfully if only a part of the whole recognized text matches the the validation pattern. If set to FALSE, the whole text must match the validation pattern. Applies to pattern validation only. Defaults to FALSE.
  bool shouldMatchSubstring;

  /// The cameras zoom level preferred for this step. The actual zoom might be different from the preferred one to avoid clipping of finder area and maintain its aspect ratio and height
  double preferredZoom;

  /// The cameras zoom level preferred for this step. The actual zoom might be different from the preferred one to avoid clipping of finder area and maintain its aspect ratio and height
  FinderAspectRatio aspectRatio;

  /// The preferred height of the finder for zoom scale 1.0 (unzoomed). The actual finder height might change to maintain aspect ratio and to not clip the screen. Defaults to 40 points.
  double unzoomedFinderHeight;

  /// A string (list) of accepted characters during text recognition. If empty or nil, all characters are accepted. Defaults to nil.
  String allowedSymbols;

  /// Recognition strategy for the text.
  TextFilterStrategy textFilterStrategy;

  /// Threshold used to pause the detection after significant movement occurred. -1 is default value. Default = 0 for textFilterStrategy='DOCUMENT' and 1000 for textFilterStrategy='LCD_DOT_MATRIX_DISPLAY'
  int significantShakeDelay;
  TextDataScannerStep(
    this.guidanceText,
    this.pattern,
    this.shouldMatchSubstring,
    this.preferredZoom,
    this.aspectRatio,
    this.unzoomedFinderHeight,
    this.allowedSymbols,
    this.textFilterStrategy,
    this.significantShakeDelay,
  );

  Map<String, dynamic> toJson() => _$TextDataScannerStepToJson(this);
}

/// Additional parameter for GenericTextRecognizer to set the type of the scanned object
enum TextFilterStrategy {
  /// Generic document printed on a paper or a card */
  Document,

  /// LCD display */
  LcdDotMatrixDisplay,
}
